/* Target definitions for arm running Darwin.
   Copyright (C) 2012
   Free Software Foundation, Inc.

This file is part of GCC.

GCC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

GCC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GCC; see the file COPYING3.  If not see
<http://www.gnu.org/licenses/>.  */

/* Enable Mach-O bits in arm code.  */
#undef TARGET_MACHO
#define TARGET_MACHO 1

#undef  DARWIN_ARM
#define DARWIN_ARM 1

#define TARGET_64BIT 0

#define DARWIN_ARCH_SPEC "armv7"

#define DARWIN_SUBARCH_SPEC "armv7"

/* crt2.o is not used.  */
#define DARWIN_CRT2_SPEC ""

#undef SUBTARGET_EXTRA_SPECS
#define SUBTARGET_EXTRA_SPECS			\
  DARWIN_EXTRA_SPECS                            \
  { "darwin_arch", DARWIN_ARCH_SPEC },		\
  { "darwin_crt2", DARWIN_CRT2_SPEC },		\
  { "darwin_subarch", DARWIN_SUBARCH_SPEC },

/* We want -fPIC by default, unless we're using -static to compile for
   the kernel or some such.  */

#undef CC1_SPEC
#define CC1_SPEC "%(cc1_cpu) \
  %{!mkernel:%{!static:%{!mdynamic-no-pic:-fPIC}}} \
  %{g: %{!fno-eliminate-unused-debug-symbols: -feliminate-unused-debug-symbols }} " \
  DARWIN_CC1_SPEC

#undef ASM_SPEC
#define ASM_SPEC "-arch %(darwin_arch) %{static}"

/* Determine a minimum version based on compiler options.  */
#define DARWIN_MINVERSION_SPEC	"7.1"

/* Use -ios_version-min on arm-darwin.  */
#undef LINK_VERSION_MIN_SPEC
#define LINK_VERSION_MIN_SPEC \
  "%{!mios-version-min=*:-ios_version_min %(darwin_minversion)} \
   %{mios-version-min=*:-ios_version_min %*}"

#define TARGET_OS_CPP_BUILTINS()                \
  do                                            \
    {                                           \
      builtin_define ("__LITTLE_ENDIAN__");     \
      darwin_cpp_builtins (pfile);		\
    }                                           \
  while (0)

#undef REAL_LIBGCC_SPEC
#define REAL_LIBGCC_SPEC "%{static-libgcc|static:-lgcc_eh -lgcc;:-lgcc_s.1}"

#undef DARWIN_CRT1_SPEC
#define DARWIN_CRT1_SPEC "-lcrt1.o %{fgnu-tm: -lcrttms.o}"

/* In iOS 3.0 and later, register R9 can be used as a volatile scratch
   register.  */
#undef  SUBTARGET_CONDITIONAL_REGISTER_USAGE
#define SUBTARGET_CONDITIONAL_REGISTER_USAGE call_used_regs[9] = 1;

/* r7 is frame pointer.  We can't change that because of builtin setjmp
   longjmp.  */
#undef  ARM_HARD_FRAME_POINTER_REGNUM
#define ARM_HARD_FRAME_POINTER_REGNUM	7

/* Default to cortex-a8.  */
#undef  SUBTARGET_CPU_DEFAULT
#define SUBTARGET_CPU_DEFAULT TARGET_CPU_cortexa8

/* Use the AAPCS ABI by default.  */
#undef  ARM_DEFAULT_ABI
#define ARM_DEFAULT_ABI ARM_ABI_AAPCS

#undef FPUTYPE_DEFAULT
#define FPUTYPE_DEFAULT "vfp"

/* Darwin use softfp.  */
#undef  TARGET_DEFAULT_FLOAT_ABI
#define TARGET_DEFAULT_FLOAT_ABI ARM_FLOAT_ABI_SOFT

/* The stack is 4-byte aligned at the point of function calls.
   Large data types (larger than 4 bytes) are 4-byte aligned.  */
#undef  ARM_DOUBLEWORD_ALIGN
#define ARM_DOUBLEWORD_ALIGN	0

/* The assembler's names for the registers.  See aout.h, except we want to
   use r11 instead of fp.  */
#undef  REGISTER_NAMES
#define REGISTER_NAMES						\
{								\
  "r0", "r1", "r2", "r3", "r4", "r5", "r6", "r7",		\
  "r8", "r9", "r10", "r11", "ip", "sp", "lr", "pc",		\
  "s0",  "s1",  "s2",  "s3",  "s4",  "s5",  "s6",  "s7",	\
  "s8",  "s9",  "s10", "s11", "s12", "s13", "s14", "s15",	\
  "s16", "s17", "s18", "s19", "s20", "s21", "s22", "s23",	\
  "s24", "s25", "s26", "s27", "s28", "s29", "s30", "s31",	\
  "d16", "?16", "d17", "?17", "d18", "?18", "d19", "?19",	\
  "d20", "?20", "d21", "?21", "d22", "?22", "d23", "?23",	\
  "d24", "?24", "d25", "?25", "d26", "?26", "d27", "?27",	\
  "d28", "?28", "d29", "?29", "d30", "?30", "d31", "?31",	\
  "wr0",   "wr1",   "wr2",   "wr3",				\
  "wr4",   "wr5",   "wr6",   "wr7",				\
  "wr8",   "wr9",   "wr10",  "wr11",				\
  "wr12",  "wr13",  "wr14",  "wr15",				\
  "wcgr0", "wcgr1", "wcgr2", "wcgr3",				\
  "cc", "vfpcc", "sfp", "afp"					\
}
