/* Extensions to the basic input location expansion services, using
   extra datastructures optionally filled by language front-ends.
   Copyright (C) 2014 Free Software Foundation, Inc.

This file is part of GCC.

GCC is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 3, or (at your option) any later
version.

GCC is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with GCC; see the file COPYING3.  If not see
<http://www.gnu.org/licenses/>.  */

#ifndef GCC_EINPUT_H
#define GCC_EINPUT_H

#include "input.h"

/* Line context information: extra data associated with source *lines* (not
   individual source locations), filled by context_expand_location from extra
   tables that front-ends may provide by the time we reach the "final"
   pass.  */

/* A pointer to a table, indexed by ordinary line-map number, where each entry
   holds a pointer to a bitmap indexed by line number within the source file
   associated with the line-map entry. Bit N in a map indicates whether source
   line number N contains multiple statements. Common bitmaps may be shared
   by line-maps corresponding to the same source file.
   

          multi_stmt_lines[]
               | |           
               ...          Does line N in source file associated
	       ...          with line map # contain multiple statements ?
               | |              v
  line-map # : |*|--o---> (--1---1----1----)
               | |  |         
	       ...  |
               | |--o  source file for this line-map entry is
	       ...     the same as for line-map # ; use a common bitmap.


  When filled, This datastructure is used by final.c to output column numbers
  on the .loc directives for insns pertaining to source lines with multiple
  statements.  We opted for a straight indexing by line-map number instead of
  a hash table keyed on source file names because there will be really lots of
  lookups.  */

extern bitmap * multi_stmt_lines;

/* For languages that have the notion of instantiating a given template
   multiple times, different linemaps can be allocated for each instance,
   which are distinguished by an instance id.  This table, which has the same
   indices as the ordinary linemaps in line_table, stores these instance
   ids.  */
extern int *instance_table;

/* The line context information per se, filled by context_expand_location
   below from data found in the tables declared above.  */

typedef struct location_lc {
  /* Whether the source line designated by the original source location
     contains multiple statements.  */
  bool multi_stmt_line;

  /* The instance id associated with the source line, if any.  */
  int instance;
} location_lc;

extern expanded_location context_expand_location (source_location loc,
						  location_lc *lcp);

#endif
